@@ -22,8 +22,8 @@ const DBus = imports.dbus;
 const Lang = imports.lang;
 const Signals = imports.signals;
 
+const AccountsService = imports.gi.AccountsService;
 const Clutter = imports.gi.Clutter;
-const Gdm = imports.gi.Gdm;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Pango = imports.gi.Pango;
@@ -60,8 +60,16 @@ const logoutDialogContent = {
     subjectWithUser: _("Log Out %s"),
     subject: _("Log Out"),
     inhibitedDescription: _("Click Log Out to quit these applications and log out of the system."),
-    uninhibitedDescriptionWithUser: _("%s will be logged out automatically in %d seconds."),
-    uninhibitedDescription: _("You will be logged out automatically in %d seconds."),
+    uninhibitedDescriptionWithUser: function(user, seconds) {
+        return ngettext("%s will be logged out automatically in %d second.",
+                        "%s will be logged out automatically in %d seconds.",
+                        seconds).format(user, seconds);
+    },
+    uninhibitedDescription: function(seconds) {
+        return ngettext("You will be logged out automatically in %d second.",
+                        "You will be logged out automatically in %d seconds.",
+                        seconds).format(seconds);
+    },
     endDescription: _("Logging out of the system."),
     confirmButtons: [{ signal: 'ConfirmedLogout',
                        label:  _("Log Out") }],
@@ -71,7 +79,11 @@ const logoutDialogContent = {
 const shutdownDialogContent = {
     subject: _("Power Off"),
     inhibitedDescription: _("Click Power Off to quit these applications and power off the system."),
-    uninhibitedDescription: _("The system will power off automatically in %d seconds."),
+    uninhibitedDescription: function(seconds) {
+        return ngettext("The system will power off automatically in %d second.",
+                        "The system will power off automatically in %d seconds.",
+                        seconds).format(seconds);
+    },
     endDescription: _("Powering off the system."),
     confirmButtons: [{ signal: 'ConfirmedReboot',
                        label:  _("Restart") },
@@ -84,7 +96,11 @@ const shutdownDialogContent = {
 const restartDialogContent = {
     subject: _("Restart"),
     inhibitedDescription: _("Click Restart to quit these applications and restart the system."),
-    uninhibitedDescription: _("The system will restart automatically in %d seconds."),
+    uninhibitedDescription: function(seconds) {
+        return ngettext("The system will restart automatically in %d second.",
+                        "The system will restart automatically in %d seconds.",
+                        seconds).format(seconds);
+    },
     endDescription: _("Restarting the system."),
     confirmButtons: [{ signal: 'ConfirmedReboot',
                        label:  _("Restart") }],
@@ -113,7 +129,7 @@ function findAppFromInhibitor(inhibitor) {
     let app = null;
     for (let i = 0; i < candidateDesktopFiles.length; i++) {
         try {
-            app = appSystem.get_app(candidateDesktopFiles[i]);
+            app = appSystem.lookup_app(candidateDesktopFiles[i]);
 
             if (app)
                 break;
@@ -173,7 +189,7 @@ ListItem.prototype = {
 
     _onClicked: function() {
         this.emit('activate');
-        this._app.activate(-1);
+        this._app.activate();
     }
 };
 Signals.addSignalMethods(ListItem.prototype);
@@ -237,7 +253,7 @@ EndSessionDialog.prototype = {
     _init: function() {
         ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: 'end-session-dialog' });
 
-        this._user = Gdm.UserManager.ref_default().get_user(GLib.get_user_name());
+        this._user = AccountsService.UserManager.get_default().get_user(GLib.get_user_name());
 
         this._secondsLeft = 0;
         this._totalSecondsToStayOpen = 0;
@@ -388,14 +404,14 @@ EndSessionDialog.prototype = {
                         subject = dialogContent.subjectWithUser.format(realName);
 
                     if (dialogContent.uninhibitedDescriptionWithUser)
-                        description = dialogContent.uninhibitedDescriptionWithUser.format(realName, displayTime);
+                        description = dialogContent.uninhibitedDescriptionWithUser(realName, displayTime);
                     else
-                        description = dialogContent.uninhibitedDescription.format(displayTime);
+                        description = dialogContent.uninhibitedDescription(displayTime);
                 }
             }
 
             if (!description)
-                description = dialogContent.uninhibitedDescription.format(displayTime);
+                description = dialogContent.uninhibitedDescription(displayTime);
         } else {
             description = dialogContent.endDescription;
         }
