@@ -40,20 +40,16 @@ LayoutManager.prototype = {
         this.panelBox.connect('allocation-changed',
                               Lang.bind(this, this._updatePanelBarriers));
 
-        // bottomBox contains the tray and keyboard (which move
-        // together, since the tray slides up from the top of the
-        // keyboard when the keyboard is visible).
-        this._bottomBox = new St.BoxLayout({ name: 'bottomBox',
-                                             vertical: true });
-        this.addChrome(this._bottomBox, { visibleInFullscreen: true });
-
         this.trayBox = new St.BoxLayout({ name: 'trayBox' }); 
+        this.addChrome(this.trayBox, { visibleInFullscreen: true });
         this.trayBox.connect('allocation-changed',
                              Lang.bind(this, this._updateTrayBarrier));
-        this._bottomBox.add_actor(this.trayBox);
 
-        this.keyboardBox = new St.BoxLayout({ name: 'keyboardBox' });
-        this._bottomBox.add_actor(this.keyboardBox);
+        this.keyboardBox = new St.BoxLayout({ name: 'keyboardBox',
+                                              reactive: true,
+                                              track_hover: true });
+        this.addChrome(this.keyboardBox, { visibleInFullscreen: true });
+        this._keyboardHeightNotifyId = 0;
 
         global.screen.connect('monitors-changed',
                               Lang.bind(this, this._monitorsChanged));
@@ -153,11 +149,13 @@ LayoutManager.prototype = {
         this.panelBox.set_position(this.primaryMonitor.x, this.primaryMonitor.y);
         this.panelBox.set_size(this.primaryMonitor.width, -1);
 
-        this._bottomBox.set_position(this.bottomMonitor.x,
-                                     this.bottomMonitor.y + this.bottomMonitor.height);
-        this._bottomBox.set_size(this.bottomMonitor.width, -1);
+        this.keyboardBox.set_position(this.bottomMonitor.x,
+                                      this.bottomMonitor.y + this.bottomMonitor.height);
+        this.keyboardBox.set_size(this.bottomMonitor.width, -1);
 
-        this.trayBox.width = this.bottomMonitor.width;
+        this.trayBox.set_position(this.bottomMonitor.x,
+                                  this.bottomMonitor.y + this.bottomMonitor.height);
+        this.trayBox.set_size(this.bottomMonitor.width, -1);
 
         // Set trayBox's clip to show things above it, but not below
         // it (so it's not visible behind the keyboard). The exact
@@ -250,23 +248,38 @@ LayoutManager.prototype = {
     },
 
     _startupAnimation: function() {
+        // Don't animate the strut
+        this._chrome.freezeUpdateRegions();
+
         this.panelBox.anchor_y = this.panelBox.height;
         Tweener.addTween(this.panelBox,
                          { anchor_y: 0,
                            time: STARTUP_ANIMATION_TIME,
-                           transition: 'easeOutQuad'
+                           transition: 'easeOutQuad',
+                           onComplete: this._startupAnimationComplete,
+                           onCompleteScope: this
                          });
     },
 
+    _startupAnimationComplete: function() {
+        this._chrome.thawUpdateRegions();
+    },
+
     showKeyboard: function () {
         Main.messageTray.hide();
-        Tweener.addTween(this._bottomBox,
-                         { anchor_y: this._bottomBox.height,
+        this.keyboardBox.raise_top();
+        Tweener.addTween(this.keyboardBox,
+                         { anchor_y: this.keyboardBox.height,
                            time: KEYBOARD_ANIMATION_TIME,
                            transition: 'easeOutQuad',
                            onComplete: this._showKeyboardComplete,
                            onCompleteScope: this
                          });
+        Tweener.addTween(this.trayBox,
+                         { anchor_y: this.keyboardBox.height,
+                           time: KEYBOARD_ANIMATION_TIME,
+                           transition: 'easeOutQuad'
+                         });
     },
 
     _showKeyboardComplete: function() {
@@ -274,20 +287,30 @@ LayoutManager.prototype = {
         // anchor point changes
         this._chrome.updateRegions();
 
-        this._bottomBox.connect('notify::height', Lang.bind(this, function () {
-            this._bottomBoxAnchor = this._bottomBox.height;
+        this._keyboardHeightNotifyId = this.keyboardBox.connect('notify::height', Lang.bind(this, function () {
+            this.keyboardBox.anchor_y = this.keyboardBox.height;
+            this.trayBox.anchor_y = this.keyboardBox.height;
         }));
     },
 
     hideKeyboard: function (immediate) {
         Main.messageTray.hide();
-        Tweener.addTween(this._bottomBox,
+        if (this._keyboardHeightNotifyId) {
+            this.keyboardBox.disconnect(this._keyboardHeightNotifyId);
+            this._keyboardHeightNotifyId = 0;
+        }
+        Tweener.addTween(this.keyboardBox,
                          { anchor_y: 0,
                            time: immediate ? 0 : KEYBOARD_ANIMATION_TIME,
                            transition: 'easeOutQuad',
-                           onComplete: this._showKeyboardComplete,
+                           onComplete: this._hideKeyboardComplete,
                            onCompleteScope: this
                          });
+        Tweener.addTween(this.trayBox,
+                         { anchor_y: 0,
+                           time: immediate ? 0 : KEYBOARD_ANIMATION_TIME,
+                           transition: 'easeOutQuad'
+                         });
     },
 
     _hideKeyboardComplete: function() {
@@ -295,11 +318,11 @@ LayoutManager.prototype = {
     },
 
     // addChrome:
-    // @actor: an actor to add to the chrome layer
+    // @actor: an actor to add to the chrome
     // @params: (optional) additional params
     //
-    // Adds @actor to the chrome layer and (unless %affectsInputRegion
-    // in @params is %false) extends the input region to include it.
+    // Adds @actor to the chrome, and (unless %affectsInputRegion in
+    // @params is %false) extends the input region to include it.
     // Changes in @actor's size, position, and visibility will
     // automatically result in appropriate changes to the input
     // region.
@@ -341,9 +364,9 @@ LayoutManager.prototype = {
     },
 
     // removeChrome:
-    // @actor: a child of the chrome layer
+    // @actor: a chrome actor
     //
-    // Removes @actor from the chrome layer
+    // Removes @actor from the chrome
     removeChrome: function(actor) {
         this._chrome.removeActor(actor);
     }
@@ -542,13 +565,10 @@ Chrome.prototype = {
     _init: function(layoutManager) {
         this._layoutManager = layoutManager;
 
-        // The group itself has zero size so it doesn't interfere with DND
-        this.actor = new Shell.GenericContainer({ width: 0, height: 0 });
-        Main.uiGroup.add_actor(this.actor);
-        this.actor.connect('allocate', Lang.bind(this, this._allocated));
-
         this._monitors = [];
         this._inOverview = false;
+        this._updateRegionIdle = 0;
+        this._freezeUpdateCount = 0;
 
         this._trackedActors = [];
 
@@ -561,6 +581,7 @@ Chrome.prototype = {
         global.screen.connect('notify::n-workspaces',
                               Lang.bind(this, this._queueUpdateRegions));
 
+        this._screenSaverActive = false;
         this._screenSaverProxy = new ScreenSaver.ScreenSaverProxy();
         this._screenSaverProxy.connect('ActiveChanged', Lang.bind(this, this._onScreenSaverActiveChanged));
         this._screenSaverProxy.GetActiveRemote(Lang.bind(this,
@@ -579,14 +600,8 @@ Chrome.prototype = {
                              Lang.bind(this, this._overviewHidden));
     },
 
-    _allocated: function(actor, box, flags) {
-        let children = this.actor.get_children();
-        for (let i = 0; i < children.length; i++)
-            children[i].allocate_preferred_size(flags);
-    },
-
     addActor: function(actor, params) {
-        this.actor.add_actor(actor);
+        Main.uiGroup.add_actor(actor);
         this._trackActor(actor, params);
     },
 
@@ -598,7 +613,7 @@ Chrome.prototype = {
             index = this._findActor(ancestor);
         }
         if (!ancestor)
-            throw new Error('actor is not a descendent of the chrome layer');
+            throw new Error('actor is not a descendent of a chrome actor');
 
         let ancestorData = this._trackedActors[index];
         if (!params)
@@ -618,7 +633,7 @@ Chrome.prototype = {
     },
 
     removeActor: function(actor) {
-        this.actor.remove_actor(actor);
+        Main.uiGroup.remove_actor(actor);
         this._untrackActor(actor);
     },
 
@@ -637,6 +652,7 @@ Chrome.prototype = {
 
         let actorData = Params.parse(params, defaultParams);
         actorData.actor = actor;
+        actorData.isToplevel = actor.get_parent() == Main.uiGroup;
         actorData.visibleId = actor.connect('notify::visible',
                                             Lang.bind(this, this._queueUpdateRegions));
         actorData.allocationId = actor.connect('notify::allocation',
@@ -666,18 +682,29 @@ Chrome.prototype = {
     },
 
     _actorReparented: function(actor, oldParent) {
-        if (!this.actor.contains(actor))
+        let newParent = actor.get_parent();
+        if (!newParent)
             this._untrackActor(actor);
+        else
+            actorData.isToplevel = (newParent == Main.uiGroup);
     },
 
     _updateVisibility: function() {
         for (let i = 0; i < this._trackedActors.length; i++) {
-            let actorData = this._trackedActors[i];
-            if (!this._inOverview && !actorData.visibleInFullscreen &&
-                this._findMonitorForActor(actorData.actor).inFullscreen)
-                this.actor.set_skip_paint(actorData.actor, true);
+            let actorData = this._trackedActors[i], visible;
+            if (!actorData.isToplevel)
+                continue;
+
+            if (this._screenSaverActive)
+                visible = false;
+            else if (this._inOverview)
+                visible = true;
+            else if (!actorData.visibleInFullscreen &&
+                     this._findMonitorForActor(actorData.actor).inFullscreen)
+                visible = false;
             else
-                this.actor.set_skip_paint(actorData.actor, false);
+                visible = true;
+            Main.uiGroup.set_skip_paint(actorData.actor, !visible);
         }
     },
 
@@ -703,7 +730,8 @@ Chrome.prototype = {
     },
 
     _onScreenSaverActiveChanged: function(proxy, screenSaverActive) {
-        this.actor.visible = !screenSaverActive;
+        this._screenSaverActive = screenSaverActive;
+        this._updateVisibility();
         this._queueUpdateRegions();
     },
 
@@ -744,11 +772,22 @@ Chrome.prototype = {
     },
 
     _queueUpdateRegions: function() {
-        if (!this._updateRegionIdle)
+        if (!this._updateRegionIdle && !this._freezeUpdateCount)
             this._updateRegionIdle = Mainloop.idle_add(Lang.bind(this, this.updateRegions),
                                                        Meta.PRIORITY_BEFORE_REDRAW);
     },
 
+    freezeUpdateRegions: function() {
+        if (this._updateRegionIdle)
+            this.updateRegions();
+        this._freezeUpdateCount++;
+    },
+
+    thawUpdateRegions: function() {
+        this._freezeUpdateCount--;
+        this._queueUpdateRegions();
+    },
+
     _updateFullscreen: function() {
         let windows = Main.getWindowActorsForWorkspace(global.screen.get_active_workspace_index());
 
@@ -756,7 +795,7 @@ Chrome.prototype = {
         for (let i = 0; i < this._monitors.length; i++)
             this._monitors[i].inFullscreen = false;
 
-        // The chrome layer should be visible unless there is a window
+        // Ordinary chrome should be visible unless there is a window
         // with layer FULLSCREEN, or a window with layer
         // OVERRIDE_REDIRECT that covers the whole screen.
         // ('override_redirect' is not actually a layer above all
@@ -837,7 +876,7 @@ Chrome.prototype = {
 
             if (actorData.affectsInputRegion &&
                 actorData.actor.get_paint_visibility() &&
-                !this.actor.get_skip_paint(actorData.actor))
+                !Main.uiGroup.get_skip_paint(actorData.actor))
                 rects.push(rect);
 
             if (!actorData.affectsStruts)
