@@ -595,9 +595,9 @@ function _globalKeyPressHandler(actor, event) {
         return true;
     }
 
-    // Other bindings are only available when the overview is up and
+    // Other bindings are only available to the user session when the overview is up and
     // no modal dialog is present.
-    if (!overview.visible || modalCount > 1)
+    if (global.session_type == Shell.SessionType.USER && (!overview.visible || modalCount > 1))
         return false;
 
     // This isn't a Meta.KeyBindingAction yet
@@ -607,7 +607,8 @@ function _globalKeyPressHandler(actor, event) {
     }
 
     if (action == Meta.KeyBindingAction.SWITCH_PANELS) {
-        ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK);
+        ctrlAltTabManager.popup(modifierState & Clutter.ModifierType.SHIFT_MASK,
+                                modifierState);
         return true;
     }
 
