@@ -2,26 +2,110 @@
 
 const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
+const Lang = imports.lang;
+const Shell = imports.gi.Shell;
 const Signals = imports.signals;
 const St = imports.gi.St;
 
-const Lang = imports.lang;
-const PopupMenu = imports.ui.popupMenu;
 const Main = imports.ui.main;
+const Params = imports.misc.params;
+const PopupMenu = imports.ui.popupMenu;
+
+function ButtonBox(params) {
+    this._init.apply(this, arguments);
+};
+
+ButtonBox.prototype = {
+    _init: function(params) {
+        params = Params.parse(params, { style_class: 'panel-button' }, true);
+        this.actor = new Shell.GenericContainer(params);
+        this.actor._delegate = this;
+
+        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        this.actor.connect('allocate', Lang.bind(this, this._allocate));
+
+        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
+        this._minHPadding = this._natHPadding = 0.0;
+    },
+
+    _onStyleChanged: function(actor) {
+        let themeNode = actor.get_theme_node();
+
+        this._minHPadding = themeNode.get_length('-minimum-hpadding');
+        this._natHPadding = themeNode.get_length('-natural-hpadding');
+    },
+
+    _getPreferredWidth: function(actor, forHeight, alloc) {
+        let children = actor.get_children();
+        let child = children.length > 0 ? children[0] : null;
+
+        if (child) {
+            [alloc.min_size, alloc.natural_size] = child.get_preferred_width(-1);
+        } else {
+            alloc.min_size = alloc.natural_size = 0;
+        }
+
+        alloc.min_size += 2 * this._minHPadding;
+        alloc.natural_size += 2 * this._natHPadding;
+    },
+
+    _getPreferredHeight: function(actor, forWidth, alloc) {
+        let children = actor.get_children();
+        let child = children.length > 0 ? children[0] : null;
+
+        if (child) {
+            [alloc.min_size, alloc.natural_size] = child.get_preferred_height(-1);
+        } else {
+            alloc.min_size = alloc.natural_size = 0;
+        }
+    },
+
+    _allocate: function(actor, box, flags) {
+        let children = actor.get_children();
+        if (children.length == 0)
+            return;
+
+        let child = children[0];
+        let [minWidth, natWidth] = child.get_preferred_width(-1);
+        let [minHeight, natHeight] = child.get_preferred_height(-1);
+
+        let availWidth = box.x2 - box.x1;
+        let availHeight = box.y2 - box.y1;
+
+        let childBox = new Clutter.ActorBox();
+        if (natWidth + 2 * this._natHPadding <= availWidth) {
+            childBox.x1 = this._natHPadding;
+            childBox.x2 = availWidth - this._natHPadding;
+        } else {
+            childBox.x1 = this._minHPadding;
+            childBox.x2 = availWidth - this._minHPadding;
+        }
+
+        if (natHeight <= availHeight) {
+            childBox.y1 = Math.floor((availHeight - natHeight) / 2);
+            childBox.y2 = childBox.y1 + natHeight;
+        } else {
+            childBox.y1 = 0;
+            childBox.y2 = availHeight;
+        }
+
+        child.allocate(childBox, flags);
+    },
+}
 
 function Button(menuAlignment) {
     this._init(menuAlignment);
 }
 
 Button.prototype = {
+    __proto__: ButtonBox.prototype,
+
     _init: function(menuAlignment) {
-        this.actor = new St.Bin({ style_class: 'panel-button',
-                                  reactive: true,
-                                  can_focus: true,
-                                  x_fill: true,
-                                  y_fill: false,
-                                  track_hover: true });
-        this.actor._delegate = this;
+        ButtonBox.prototype._init.call(this, { reactive: true,
+                                               can_focus: true,
+                                               track_hover: true });
+
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
         this.actor.connect('key-press-event', Lang.bind(this, this._onSourceKeyPress));
         this.menu = new PopupMenu.PopupMenu(this.actor, menuAlignment, St.Side.TOP);
@@ -112,7 +196,8 @@ SystemStatusButton.prototype = {
         this._iconActor = new St.Icon({ icon_name: iconName,
                                         icon_type: St.IconType.SYMBOLIC,
                                         style_class: 'system-status-icon' });
-        this.actor.set_child(this._iconActor);
+        this.actor.add_actor(this._iconActor);
+        this.actor.add_style_class_name('panel-status-button');
         this.setTooltip(tooltipText);
     },
 
