@@ -35,6 +35,7 @@ PopupBaseMenuItem.prototype = {
         params = Params.parse (params, { reactive: true,
                                          activate: true,
                                          hover: true,
+                                         sensitive: true,
                                          style_class: null
                                        });
         this.actor = new Shell.GenericContainer({ style_class: 'popup-menu-item',
@@ -52,11 +53,15 @@ PopupBaseMenuItem.prototype = {
         this._columnWidths = null;
         this._spacing = 0;
         this.active = false;
+        this._activatable = params.reactive && params.activate;
+        this.sensitive = this._activatable && params.sensitive;
+
+        this.setSensitive(this.sensitive);
 
         if (params.style_class)
             this.actor.add_style_class_name(params.style_class);
 
-        if (params.reactive && params.activate) {
+        if (this._activatable) {
             this.actor.connect('button-release-event', Lang.bind(this, this._onButtonReleaseEvent));
             this.actor.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
         }
@@ -117,6 +122,23 @@ PopupBaseMenuItem.prototype = {
         }
     },
 
+    setSensitive: function(sensitive) {
+        if (!this._activatable)
+            return;
+        if (this.sensitive == sensitive)
+            return;
+
+        this.sensitive = sensitive;
+        this.actor.reactive = sensitive;
+        this.actor.can_focus = sensitive;
+
+        if (sensitive)
+            this.actor.remove_style_pseudo_class('insensitive');
+        else
+            this.actor.add_style_pseudo_class('insensitive');
+        this.emit('sensitive-changed', sensitive);
+    },
+
     destroy: function() {
         this.actor.destroy();
         this.emit('destroy');
@@ -222,10 +244,25 @@ PopupBaseMenuItem.prototype = {
     },
 
     _getPreferredHeight: function(actor, forWidth, alloc) {
-        let height = 0;
+        let height = 0, x = 0, minWidth, childWidth;
         for (let i = 0; i < this._children.length; i++) {
             let child = this._children[i];
-            let [min, natural] = child.actor.get_preferred_height(forWidth);
+            if (this._columnWidths) {
+                if (child.span == -1) {
+                    childWidth = 0;
+                    for (let j = i; j < this._columnWidths.length; j++)
+                        childWidth += this._columnWidths[j]
+                } else
+                    childWidth = this._columnWidths[i];
+            } else {
+                if (child.span == -1)
+                    childWidth = forWidth - x;
+                else
+                    [minWidth, childWidth] = child.actor.get_preferred_width(-1);
+            }
+            x += childWidth;
+
+            let [min, natural] = child.actor.get_preferred_height(childWidth);
             if (natural > height)
                 height = natural;
         }
@@ -283,7 +320,14 @@ PopupBaseMenuItem.prototype = {
                 }
                 extraWidth = availWidth - naturalWidth;
             } else {
-                availWidth = naturalWidth;
+                if (child.span == -1) {
+                    if (direction == St.TextDirection.LTR)
+                        availWidth = box.x2 - x;
+                    else
+                        availWidth = x - box.x1;
+                } else {
+                    availWidth = naturalWidth;
+                }
                 extraWidth = 0;
             }
 
@@ -291,7 +335,7 @@ PopupBaseMenuItem.prototype = {
                 if (child.expand) {
                     childBox.x1 = x;
                     childBox.x2 = x + availWidth;
-                } else if (child.align === St.Align.CENTER) {
+                } else if (child.align === St.Align.MIDDLE) {
                     childBox.x1 = x + Math.round(extraWidth / 2);
                     childBox.x2 = childBox.x1 + naturalWidth;
                 } else if (child.align === St.Align.END) {
@@ -305,7 +349,7 @@ PopupBaseMenuItem.prototype = {
                 if (child.expand) {
                     childBox.x1 = x - availWidth;
                     childBox.x2 = x;
-                } else if (child.align === St.Align.CENTER) {
+                } else if (child.align === St.Align.MIDDLE) {
                     childBox.x1 = x - Math.round(extraWidth / 2);
                     childBox.x2 = childBox.x1 + naturalWidth;
                 } else if (child.align === St.Align.END) {
@@ -899,6 +943,17 @@ PopupMenuBase.prototype = {
                 this.emit('active-changed', null);
             }
         }));
+        menuItem._sensitiveChangeId = menuItem.connect('sensitive-changed', Lang.bind(this, function(menuItem, sensitive) {
+            if (!sensitive && this._activeMenuItem == menuItem) {
+                if (!this.actor.navigate_focus(menuItem.actor,
+                                               Gtk.DirectionType.TAB_FORWARD,
+                                               true))
+                    this.actor.grab_key_focus();
+            } else if (sensitive && this._activeMenuItem == null) {
+                if (global.stage.get_key_focus() == this.actor)
+                    menuItem.actor.grab_key_focus();
+            }
+        }));
         menuItem._activateId = menuItem.connect('activate', Lang.bind(this, function (menuItem, event) {
             this.emit('activate', menuItem);
             this.close(true);
@@ -906,6 +961,7 @@ PopupMenuBase.prototype = {
         menuItem.connect('destroy', Lang.bind(this, function(emitter) {
             menuItem.disconnect(menuItem._activateId);
             menuItem.disconnect(menuItem._activeChangeId);
+            menuItem.disconnect(menuItem._sensitiveChangeId);
             if (menuItem.menu) {
                 menuItem.menu.disconnect(menuItem._subMenuActivateId);
                 menuItem.menu.disconnect(menuItem._subMenuActiveChangeId);
@@ -1528,7 +1584,8 @@ PopupComboBoxMenuItem.prototype = {
         this.addActor(this._itemBox);
 
         let expander = new St.Label({ text: '\u2304' });
-        this.addActor(expander, { align: St.Align.END });
+        this.addActor(expander, { align: St.Align.END,
+                                  span: -1 });
 
         this._menu = new PopupComboMenu(this.actor);
         Main.uiGroup.add_actor(this._menu.actor);
