@@ -46,8 +46,12 @@ const GnomeShellIface = {
                 outSignature: ''
               },
               { name: 'InstallRemoteExtension',
-                inSignature: 's',
+                inSignature: 'ss',
                 outSignature: ''
+              },
+              { name: 'UninstallExtension',
+                inSignature: 's',
+                outSignature: 'b'
               }
              ],
     signals: [{ name: 'ExtensionStatusChanged',
@@ -174,8 +178,12 @@ GnomeShell.prototype = {
         global.settings.set_strv(ExtensionSystem.ENABLED_EXTENSIONS_KEY, enabledExtensions);
     },
 
-    InstallRemoteExtension: function(uuid, url) {
-        ExtensionSystem.installExtensionFromManifestURL(uuid, url);
+    InstallRemoteExtension: function(uuid, version_tag) {
+        ExtensionSystem.installExtensionFromUUID(uuid, version_tag);
+    },
+
+    UninstallExtension: function(uuid) {
+        return ExtensionSystem.uninstallExtensionFromUUID(uuid);
     },
 
     get OverviewActive() {
@@ -189,7 +197,7 @@ GnomeShell.prototype = {
             Main.overview.hide();
     },
 
-    ApiVersion: 1,
+    ApiVersion: ExtensionSystem.API_VERSION,
 
     ShellVersion: Config.PACKAGE_VERSION,
 
