@@ -83,6 +83,9 @@ Client.prototype = {
         this._chatSources = {};
         this._chatState = Tp.ChannelChatState.ACTIVE;
 
+        // account path -> AccountNotification
+        this._accountNotifications = {};
+
         // Set up a SimpleObserver, which will call _observeChannels whenever a
         // channel matching its filters is detected.
         // The second argument, recover, means _observeChannels will be run
@@ -302,6 +305,8 @@ Client.prototype = {
         Shell.get_tp_contacts(conn, [targetHandle],
                 contactFeatures,
                 Lang.bind(this, this._createAudioVideoSource, channel, context, dispatchOp));
+
+        context.delay();
     },
 
     _createAudioVideoSource: function(connection, contacts, failed, channel, context, dispatchOp) {
@@ -335,6 +340,8 @@ Client.prototype = {
         Shell.get_tp_contacts(conn, [targetHandle],
                 contactFeatures,
                 Lang.bind(this, this._createFileTransferSource, channel, context, dispatchOp));
+
+        context.delay();
     },
 
     _createFileTransferSource: function(connection, contacts, failed, channel, context, dispatchOp) {
@@ -420,7 +427,6 @@ Client.prototype = {
 
         /* Display notification to ask user to accept/reject request */
         let source = this._ensureSubscriptionSource();
-        Main.messageTray.add(source);
 
         let notif = new SubscriptionRequestNotification(source, contact);
         source.notify(notif);
@@ -430,6 +436,7 @@ Client.prototype = {
         if (this._subscriptionSource == null) {
             this._subscriptionSource = new MultiNotificationSource(
                 _("Subscription request"), 'gtk-dialog-question');
+            Main.messageTray.add(this._subscriptionSource);
             this._subscriptionSource.connect('destroy', Lang.bind(this, function () {
                 this._subscriptionSource = null;
             }));
@@ -446,11 +453,18 @@ Client.prototype = {
             return;
         }
 
+        let notif = this._accountNotifications[account.get_object_path()];
+        if (notif)
+            return;
+
         /* Display notification that account failed to connect */
         let source = this._ensureAccountSource();
-        Main.messageTray.add(source);
 
-        let notif = new AccountNotification(source, account, connectionError);
+        notif = new AccountNotification(source, account, connectionError);
+        this._accountNotifications[account.get_object_path()] = notif;
+        notif.connect('destroy', Lang.bind(this, function() {
+            delete this._accountNotifications[account.get_object_path()];
+        }));
         source.notify(notif);
     },
 
@@ -458,6 +472,7 @@ Client.prototype = {
         if (this._accountSource == null) {
             this._accountSource = new MultiNotificationSource(
                 _("Connection error"), 'gtk-dialog-error');
+            Main.messageTray.add(this._accountSource);
             this._accountSource.connect('destroy', Lang.bind(this, function () {
                 this._accountSource = null;
             }));
@@ -532,13 +547,6 @@ ChatSource.prototype = {
     createNotificationIcon: function() {
         this._iconBox = new St.Bin({ style_class: 'avatar-box' });
         this._iconBox._size = this.ICON_SIZE;
-
-        this._updateAvatarIcon();
-
-        return this._iconBox;
-    },
-
-    _updateAvatarIcon: function() {
         let textureCache = St.TextureCache.get_default();
         let file = this._contact.get_avatar_file();
 
@@ -550,6 +558,13 @@ ChatSource.prototype = {
                                                 icon_type: St.IconType.FULLCOLOR,
                                                 icon_size: this._iconBox._size });
         }
+
+        return this._iconBox;
+    },
+
+    _updateAvatarIcon: function() {
+        this._setSummaryIcon(this.createNotificationIcon());
+        this._notification.update(this._notification.title, null, { customContent: true, icon: this.createNotificationIcon() });
     },
 
     open: function(notification) {
@@ -1448,17 +1463,12 @@ AccountNotification.prototype = {
             _("Connection to %s failed").format(account.get_display_name()),
             null, { customContent: true });
 
-        let message;
-        if (connectionError in _connectionErrorMessages) {
-            message = _connectionErrorMessages[connectionError];
-        } else {
-            message = _("Unknown reason");
-        }
+        this._label = new St.Label();
+        this.addActor(this._label);
+        this._updateMessage(connectionError);
 
         this._account = account;
 
-        this.addBody(message);
-
         this.addButton('reconnect', _("Reconnect"));
         this.addButton('edit', _("Edit account"));
 
@@ -1492,11 +1502,25 @@ AccountNotification.prototype = {
 
         this._connectionStatusId = account.connect('notify::connection-status',
             Lang.bind(this, function() {
-                if (account.connection_status != Tp.ConnectionStatus.DISCONNECTED)
+                let status = account.connection_status;
+                if (status == Tp.ConnectionStatus.CONNECTED) {
                     this.destroy();
+                } else if (status == Tp.ConnectionStatus.DISCONNECTED) {
+                    this._updateMessage(account.connection_error);
+                }
             }));
     },
 
+    _updateMessage: function(connectionError) {
+        let message;
+        if (connectionError in _connectionErrorMessages) {
+            message = _connectionErrorMessages[connectionError];
+        } else {
+            message = _("Unknown reason");
+        }
+        this._label.set_text(message);
+    },
+
     destroy: function() {
         if (this._enabledId != 0) {
             this._account.disconnect(this._enabledId);
